/*
 * Copyright (c) 2017 OnGres, Inc.
 * SPDX-License-Identifier: BSD-2-Clause
 */

/**
 * This package expose the exceptions that can be throw by the client/server
 * implementations of SCRAM.
 */

package com.ongres.scram.common.exception;
