/*
 * Copyright (c) 2017 OnGres, Inc.
 * SPDX-License-Identifier: BSD-2-Clause
 */

/**
 * This module expose the client implementation of Salted Challenge Response
 * Authentication Mechanism (SCRAM). It provides a high level and easy to use API to negotiate the
 * mechanism and the message flow used for authentication.
 */

package com.ongres.scram.client;
