/*
 * Copyright (c) 2017 OnGres, Inc.
 * SPDX-License-Identifier: BSD-2-Clause
 */

/**
 * This package expose the messages used to implement a client/server of Salted Challenge Response
 * Authentication Mechanism (SCRAM).
 *
 * @implNote {@code extensions} are not supported.
 */

package com.ongres.scram.common;
